clear;
close all;
global vyt
load sp500.txt -ascii;  % Daily data from 12/30/1994 to 3/31/2008
%
r=100*log(sp500(2:end)./sp500(1:end-1)); vyt=r;
T=length(r);
mu=mean(r);
vsigma2t=(r-mu).^2;
figure;
subplot(2,1,1);
plot(vyt);
title('\fontsize{16} Percentage change in S&P500');
subplot(2,1,2);
plot(vsigma2t);
title('\fontsize{16} Squared yields');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Frequentist estimation
%
mu0=mu;
delta10=0.3;
alpha10=0.2;
delta10tr=log(delta10/(1-delta10-alpha10));
alpha10tr=log(alpha10/(1-alpha10-delta10));
k0=(1-delta10-alpha10)*0.4;  %*std(vyt)^2;
k0tr=log(k0);
th0tr=[mu0,k0tr,delta10tr,alpha10tr];
options=optimset('Display','iter','MaxFunEvals',10000,...
    'MaxIter',10000,'TolFun',0.00001);
thtropt=fminsearch('lfv1c',th0tr,options,1); %thtropt=fminunc('lfv',thtr0,options);
muopt=thtropt(1);
delta1opt=exp(thtropt(3))/(1+exp(thtropt(4))+exp(thtropt(3)));
alpha1opt=exp(thtropt(4))/(1+exp(thtropt(3))+exp(thtropt(4)));
kopt=exp(thtropt(2));
[lfv,vsigma2]=lfv1c(thtropt,1);
figure;
x=1:1:length(vsigma2);
p=plot(x,vsigma2t,x,vsigma2,'k');
p(2).LineWidth=2;
legend({'Squared Yields', 'Frequentist Estimation of Volatility'},'FontSize',14);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
thopt=[muopt kopt delta1opt alpha1opt]'; 
x0=thopt; n=length(x0);
    % Computing the Hessian numerically
    H0=zeros(n,n);
    auxi=1e-5*diag(x0);
    for i=1:n
        for j=1:n
            H0(i,j)=(feval('lfv1c',x0+auxi(:,i)+auxi(:,j),0)-...
                        feval('lfv1c',x0+auxi(:,i),0)-...
                        feval('lfv1c',x0+auxi(:,j),0)+...
                        feval('lfv1c',x0,0))/(auxi(j,j)^2);
        end
    end
%    
informd=inv(H0);
sgd=sqrt(diag(informd)); % standard deviations for the original parameters
%%%%%%%%%
% Computing numerically the Hessian of the transformed vector of parameters 
%
    H0t=zeros(n,n);
    x0t=thtropt';
    auxi=1e-5*diag(abs(x0t));
    for i=1:n
        for j=1:n
            H0t(i,j)=(feval('lfv1c',x0t+auxi(:,i)+auxi(:,j),1)-...
                        feval('lfv1c',x0t+auxi(:,i),1)-...
                        feval('lfv1c',x0t+auxi(:,j),1)+...
                        feval('lfv1c',x0t,1))/(auxi(j,j)^2);
        end
    end
MCOVt=inv(H0t);  % Covariance matrix of the transformed parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bayesian estimation  (MH algorithm with random walk)
% Random walk chain Metropolis-Hastings algorithm
% NOTE: We estimate the transformed parameters
N=30000;       % number of realizations for the posterior distribution
Nprior=30000;  % number of realizations for the prior  distribution
dN=0.3*N;      % burn-in
vquant=[0.025, 0.975]; % Quantiles to be computed 
c=0.1;         % the weight parameter for the covariance matrix of the parameters 
               % to be estimated
Msigma=c*diag(diag(MCOVt));  % Covariance matrix of the transformed parameters
Pchol=chol(Msigma);
% Realizations of the prior distributions of the parameters to be estimated
% without transformation. We keep the average of the realizations 
% Initial condition to strat the Metropolis-Hastings algorithm
% with random walk
hyp_mu=[x0(1)*1.2,sgd(1)];       % hyperparameter for mu (Normal distribution)
hyp_k=[x0(2)/0.01,0.01];         % hiperparameters for k (Gamma distribution)
hyp_alpha_delta=[2*x0(4)/(1-x0(4)-x0(3)),2*x0(3)/(1-x0(4)-x0(3)),4]; % hiperparameteres for alpha1 y delta1 (Dirichlet distribution
%
% Random number generation to graph the prior and posterior distributions
%
pmu_prior=normrnd(hyp_mu(1),hyp_mu(2),Nprior,1); 
pk_prior=gamrnd(hyp_k(1),hyp_k(2),Nprior,1);
    D_prior=drchrnd(hyp_alpha_delta,Nprior);
pdelta_prior=D_prior(:,2);
palpha_prior=D_prior(:,1);
paramprior=[pmu_prior pk_prior pdelta_prior palpha_prior];   % we keep these realizations to compute histograms 
thetab0=[mean(pmu_prior) mean(pk_prior) mean(pdelta_prior) mean(palpha_prior)]';
% Notice that we estimate and draw realizations of the transformed
% parameters, to guarantee that they belong to the region of admissible values
%
thetabtr0=[thetab0(1);
           log(thetab0(2));
           log(thetab0(3)/(1-thetab0(4)-thetab0(3)));
           log(thetab0(4)/(1-thetab0(3)-thetab0(4)))];  % parmetros transformados
%
% We compute the log of the joint density for the 4 parameters
%
lnprior0=(-0.5*log(hyp_mu(2))-0.5*((thetab0(1)-hyp_mu(1))/hyp_mu(2))^2)+...
         (hyp_k(1)-1)*log(thetab0(2))-thetab0(2)/hyp_k(2)+...
         (hyp_alpha_delta(1)-1)*log(thetab0(4))+...
         (hyp_alpha_delta(2)-1)*log(thetab0(3))+...
         (hyp_alpha_delta(3)-1)*log(1-thetab0(3)-thetab0(4));
%
% We compute the derivatives of the transformed parameters (in logs) 
%
lnjacob=log(thetab0(2))+log(1-thetab0(3))+log(thetab0(3))+log(1-thetab0(4))+log(thetab0(4));          
%
% We compute the prior distribution of the transformed variable 
%
lnprior0=lnprior0+lnjacob;
%
% We compute the likelihood of the transformed parameters
%
lfvp0=-lfv1c(thetab0,0);
Prob_Acept=0;   %  We initialize the acceptance probability at zero
postdraws=zeros(N+dN,2*length(thetab0)+5); % We save the posterior distribution for the parameters, the acceptance probability the priori densities and the likelihoods
%
N_Prob_Acept=0; % we initialize the number of acceptances at zero, and accumulate 
timer=clock;    % we start the clock at zero to count the time it takes to compute the posterior distribution
% Metropolis-Hastings Algorithm
for i=1:N+dN
    % take a candidate draw
    eps=randn(length(thetab0),1);
    eps1=Pchol'*eps;
    thetabtr1=thetabtr0+eps1;
    thetab1=[thetabtr1(1);
             exp(thetabtr1(2));
             exp(thetabtr1(3))/(1+exp(thetabtr1(4))+exp(thetabtr1(3)));
             exp(thetabtr1(4))/(1+exp(thetabtr1(4))+exp(thetabtr1(3)))];
    lnprior1=(-0.5*log(hyp_mu(2))-0.5*((thetab1(1)-hyp_mu(1))/hyp_mu(2))^2)+...
         (hyp_k(1)-1)*log(thetab1(2))-thetab1(2)/hyp_k(2)+...
         (hyp_alpha_delta(1)-1)*log(thetab1(4))+...
         (hyp_alpha_delta(2)-1)*log(thetab1(3))+...
         (hyp_alpha_delta(3)-1)*log(1-thetab1(3)-thetab1(4));
    lnjacob=log(thetab1(2))+log(thetab1(3))+log(thetab1(4))+log(1-thetab1(4)-thetab1(3));
    lnprior1=lnprior1+lnjacob;
    lfvp1=-lfv1c(thetab1,0);
    Prob_Acept=exp(lnprior1+lfvp1-lnprior0-lfvp0);
    U=rand(1,1)<Prob_Acept;
    N_Prob_Acept=N_Prob_Acept+U;
    thetab0=U*thetab1+(1-U)*thetab0;
    thetabtr0=U*thetabtr1+(1-U)*thetabtr0;
    lnprior0=U*lnprior1+(1-U)*lnprior0;
    lfvp0=U*lfvp1+(1-U)*lfvp0;
    postdraws(i,:)=[thetab0' thetabtr0' Prob_Acept lnprior0 lnprior1 lfvp0 lfvp1];
    [i/(N+dN)*100 N_Prob_Acept/i]
end
disp('Time elapsed in generating the posterior');
timer=etime(clock,timer)
disp('Aceptance probability');
N_Prob_Acept=N_Prob_Acept/(N+dN)
postdraws=postdraws(dN+1:N+dN,:);
V=cov(postdraws(:,length(thetab0)+1:2*length(thetab0)));
%
figure;
subplot(2,2,1);
    h1=histogram(paramprior(:,1));
    h1.FaceColor='y';
    h1.EdgeColor='none';
    h1.Normalization='pdf';
    hold on
    h2=histogram(postdraws(:,1));
    h2.FaceColor='b';
    h2.EdgeColor='none';
    h2.Normalization='pdf';
    title('\fontsize{16}\mu');
    v=axis;
    line(mean(postdraws(:,1))*ones(1,2),[v(3) v(4)],'Linewidth',3,'Color','b');
    line(mean(paramprior(:,1))*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','y');
    line(muopt*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','k');
    legend('prior','posterior','mean-posterior','mean-prior','frequentist estimation');
    hold off
subplot(2,2,2);
    h1=histogram(paramprior(:,2));
    h1.FaceColor='y';
    h1.EdgeColor='none';
    h1.Normalization='pdf';
    hold on
    h2=histogram(postdraws(:,2));
    h2.FaceColor='b';
    h2.EdgeColor='none';
    h2.Normalization='pdf';
    title('\fontsize{16}\itk');
    v=axis;
    line(mean(postdraws(:,2))*ones(1,2),[v(3) v(4)],'Linewidth',3,'Color','b');
    line(mean(paramprior(:,2))*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','y');
    line(kopt*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','k');
    legend('prior','posterior','mean-posterior','mean-prior','frequentist estimation');
    hold off
subplot(2,2,3);
    h1=histogram(paramprior(:,3));
    h1.FaceColor='y';
    h1.EdgeColor='none';
    h1.Normalization='pdf';
    hold on
    h2=histogram(postdraws(:,3));
    h2.FaceColor='b';
    h2.EdgeColor='none';
    h2.Normalization='pdf';
    title('\fontsize{16}\it\delta');
    v=axis;
    line(mean(postdraws(:,3))*ones(1,2),[v(3) v(4)],'Linewidth',3,'Color','b');
    line(mean(paramprior(:,3))*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','y');
    line(delta1opt*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','k');
    legend('prior','posterior','mean-posterior','mean-prior','frequentist estimation');
    hold off
subplot(2,2,4);
    h1=histogram(paramprior(:,4));
    h1.FaceColor='y';
    h1.EdgeColor='none';
    h1.Normalization='pdf';
    hold on
    h2=histogram(postdraws(:,4));
    h2.FaceColor='b';
    h2.EdgeColor='none';
    h2.Normalization='pdf';
    title('\fontsize{16}\it\alpha');
    v=axis;
    line(mean(postdraws(:,4))*ones(1,2),[v(3) v(4)],'Linewidth',3,'Color','b');
    line(mean(paramprior(:,4))*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','y');
    line(alpha1opt*ones(1,2),[v(3) v(4)],'Linewidth',2,'Color','k');
    legend('prior','posterior','mean-posterior','mean-prior','frequentist estimation');
    hold off
%
format short
names_var={' mu ';' k ';' delta ';' alpha '};
freq_estimation=x0;
freq_Std=sgd;
mean_posterior=mean(postdraws(:,1:4))';
perc025=prctile(postdraws(:,1:4),vquant(1)*100);   
perc975=prctile(postdraws(:,1:4),vquant(2)*100);
percentile02_5=perc025';
percentile97_5=perc975';
disp('Frequentist estimation and Bayesian Inference')
TABLE=table(freq_estimation,freq_Std,mean_posterior,percentile02_5,percentile97_5,...
            'RowNames',names_var)
    